import os

import pandas

from pm4ngs.jupyterngsplugin.markdown.utils import get_link_image


def print_tools(c, tools, files, data_path, width, height):
    str_msg = '| '
    for t in tools:
        f = [f for f in files if c + '_' + t + '_volcano.pdf' in f]
        if len(f) == 1:
            f = os.path.relpath(os.path.join(data_path, f[0]))
            str_msg += get_link_image(f, width, height, ' --- ')
            str_msg += ' | '
        else:
            str_msg += '| --- |'
    str_msg += '\n| '
    for t in tools:
        f = [f for f in files if c + '_' + t + '_correlation_heatmap' in f]
        if len(f) == 1:
            f = os.path.relpath(os.path.join(data_path, f[0]))
            str_msg += get_link_image(f, width, height, ' --- ')
            str_msg += ' | '
        else:
            str_msg += ' --- |'
    str_msg += '\n| '
    for t in tools:
        f = [f for f in files if c + '_' + t + '_expression_heatmap.pdf' in f]
        if len(f) == 1:
            f = os.path.relpath(os.path.join(data_path, f[0]))
            str_msg += get_link_image(f, width, height, ' --- ')
            str_msg += ' | '
        else:
            str_msg += ' --- |'
    str_msg += '\n| '
    for t in tools:
        f = [f for f in files if c + '_' + t + '_pca.pdf' in f]
        if len(f) == 1:
            f = os.path.relpath(os.path.join(data_path, f[0]))
            str_msg += get_link_image(f, width, height, ' --- ')
            str_msg += ' | '
        else:
            str_msg += ' --- |'
    return str_msg


def dga_table(conditions, tools, data_path, width, height):
    str_msg = 'Click on figure to retrieve original PDF file\n\n'

    for c in conditions:
        files = [f for d, ds, files in os.walk(data_path) for f in files if c in f]
        str_msg += '#### Condition: ' + c.replace('_', ' ') + '\n\n'
        for t in tools:
            str_msg += '| ' + tools[t] + ' '
        str_msg += '|\n'
        for t in tools:
            str_msg += '| --- '
        str_msg += '|\n'

        str_msg += print_tools(c, tools, files, data_path, width, height)
        str_msg += '\n\n'
    return str_msg


def dga_gene_list_union(conditions, data_path, organism):
    str_msg = ''
    table_header = '''
    <table>
    <thead>
    <tr>
    <th>Gene</th>
    <th>logFC</th>
    <th>FDR</th>
    </tr>
    </thead>
    <tbody>
    '''
    for c in conditions:
        f = os.path.relpath(os.path.join(data_path,
                                         'condition_' + c + '_union.csv'))
        o = os.path.relpath(os.path.join(data_path,
                                         'condition_' + c + '_union_over-expressed.csv'))
        u = os.path.relpath(os.path.join(data_path,
                                         'condition_' + c + '_union_under-expressed.csv'))
        if os.path.exists(f) and os.path.getsize(f) != 0:
            str_msg += '\n\n### Condition: ' + c.replace('_vs_', ' vs ') + '\n\n'
            str_msg += 'Full list of genes <a href="'
            str_msg += f.replace(' ', '%20')
            str_msg += '" target="_blank">'
            str_msg += os.path.basename(f)
            str_msg += '</a>\n\n'
            str_msg += '| Genes over-expressed | Genes under-expressed |\n'
            str_msg += '| --- | --- |\n'

            over_df = pandas.read_csv(o)
            if len(over_df) > 0:
                over_df[['Gene_Id', 'Chr', 'Start_pos']] = \
                    over_df['Gene_Id'].str.split('_', n=2, expand=True)
                over_df = over_df.drop(columns=['Chr', 'Start_pos'])
            over_count = len(over_df)
            str_msg += ' | ' + str(over_count) + ' | '

            under_df = pandas.read_csv(u)
            if len(under_df) > 0:
                under_df[['Gene_Id', 'Chr', 'Start_pos']] = \
                    under_df['Gene_Id'].str.split('_', n=2, expand=True)
                under_df = under_df.drop(columns=['Chr', 'Start_pos'])
            under_count = len(under_df)
            str_msg += str(under_count) + ' |\n\n'
            if under_count > 0 or over_count > 0:
                str_msg += '<div style="display: table;width: 100%;">'
                str_msg += '<div style="display: table-row;">'
                str_msg += '<div style="display: table-cell;vertical-align:top;">\n\n'
                if len(over_df) > 0:
                    str_msg += '<h3>Top 30 Over-expressed genes</h3>'
                    str_msg += '<br>CSV file:<br><a href="'
                    str_msg += o.replace(' ', '%20')
                    str_msg += '" target="_blank">'
                    str_msg += os.path.basename(o)
                    str_msg += '</a><br>'
                    str_msg += table_header
                    for i, r in over_df.head(30).iterrows():
                        str_msg += '<tr>'
                        str_msg += '<td>'
                        str_msg += '<a href="https://www.ncbi.nlm.nih.gov/gene/?term=' \
                                   + r['Gene_Id'] + '%5BGene+Name%5D+AND+' \
                                   + organism.replace(' ', '+') \
                                   + '%5BOrganism%5D" target="_blank">' \
                                   + r['Gene_Id'] + '</a>'
                        str_msg += '</td>'
                        str_msg += '<td>' + "{:.3f}".format(r['logFC']) + '</td>'
                        str_msg += '<td>' + "{:.3e}".format(r['FDR']) + '</td>'
                        str_msg += '</tr>'
                    str_msg += '</tbody></table>'
                str_msg += '</div>'
                str_msg += '<div style="display: table-cell;vertical-align:top;">\n\n'
                if len(under_df) > 0:
                    str_msg += '<h3>Top 30 Under-expressed genes</h3>'
                    str_msg += '<br>CSV file:<br><a href="'
                    str_msg += u.replace(' ', '%20')
                    str_msg += '" target="_blank">'
                    str_msg += os.path.basename(u)
                    str_msg += '</a><br>'
                    str_msg += table_header
                    for i, r in under_df.head(30).iterrows():
                        str_msg += '<tr>'
                        str_msg += '<td>'
                        str_msg += '<a href="https://www.ncbi.nlm.nih.gov/gene/?term=' \
                                   + r['Gene_Id'] + '%5BGene+Name%5D+AND+' \
                                   + organism.replace(' ', '+') \
                                   + '%5BOrganism%5D" target="_blank">' \
                                   + r['Gene_Id'] + '</a>'
                        str_msg += '</td>'
                        str_msg += '<td>' + "{:.3f}".format(r['logFC']) + '</td>'
                        str_msg += '<td>' + "{:.3e}".format(r['FDR']) + '</td>'
                        str_msg += '</tr>'
                    str_msg += '</tbody></table>'
                str_msg += '</div>'
                str_msg += '</div>'
                str_msg += '</div>'
    return str_msg



import base64
import io

import pdf2image
from PIL import Image


def piltobase64(pil_image):
    stream = io.BytesIO()
    pil_image.save(stream, format="PNG")
    return base64.b64encode(stream.getvalue()).decode('utf-8')


def pdftobase64(pdf_file, width, height):
    """
    Return base64 image string image from a PDF file to be used in HTML code
    :param pdf_file_name: PDF file name
    :param width: Final image width
    :param height: Final image height
    :return: base64 image string
    """
    pil_images = pdf2image.convert_from_path(pdf_file, dpi=300,
                                             output_folder=None, first_page=None,
                                             last_page=None, fmt='PNG', thread_count=1,
                                             userpw=None, use_cropbox=False, strict=False)
    pil_image = pil_images[0].resize((width, height), Image.ANTIALIAS)
    return piltobase64(pil_image)



import os
import numpy as np
import matplotlib.pyplot as plt

from pm4ngs.jupyterngsplugin.markdown.utils import find_file_print_link_size, get_link_size
from pm4ngs.jupyterngsplugin.utils.load_content_dict import load_content_dict
from pm4ngs.jupyterngsplugin.utils.load_content_dict import load_content_dict_line


def star_alignment_table_single(samples_data, sample_list, alignment_path):
    """
    Print alignment report table parsing data from STAR
    :param samples_data: Dict returned from fastqc_trimmomatic_table with extracted data
    :param sample_list: list of samples prefix names
    :param alignment_path: Folder to alignment results
    :return: Dict with samples data and the table in string format
    """
    str_msg = '| Sample | Sorted<br>BAM | BAM<br>Index | STATS'
    str_msg += '| Number<br>of input<br>reads | Uniquely<br>mapped<br>reads % '
    str_msg += '| % of reads<br>mapped to<br>multiple<br>loci '
    str_msg += '| Number of<br>splices:<br>Annotated<br>(sjdb) '
    str_msg += '| Mismatch<br>rate per<br>base, % '
    str_msg += '|\n| --- | --- | --- | --- '
    str_msg += '| --- | --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '|\n'
    for s in sample_list:
        str_msg += '| ' + s + ' | '
        str_msg += find_file_print_link_size(alignment_path, s, '_sorted.bam', 'MB', ' --- ')
        str_msg += ' |'
        str_msg += find_file_print_link_size(alignment_path, s, '_sorted.bam.bai', 'MB', ' --- ')
        str_msg += ' |'
        files = [f for ds, dr, files in os.walk(alignment_path) for f in files if
                 f.startswith(s) and f.endswith('Log.final.out')
                 and os.path.getsize(os.path.join(alignment_path, f)) != 0]
        if len(files) == 1:
            f = os.path.relpath(os.path.join(alignment_path, files[0]))
            str_msg += get_link_size(f, 'KB', ' --- ')
            str_msg += ' |'
            stats = load_content_dict(f, '\t', True, ' |', '')
            samples_data[s]['alignment'] = {
                'unique': int(stats['Uniquely mapped reads number']),
                'multiple': int(stats['Number of reads mapped to multiple loci']),
                'unmapped':
                    int(stats['Number of input reads'])
                    - (int(stats['Uniquely mapped reads number'])
                       + int(stats['Number of reads mapped to multiple loci']))
            }
            str_msg += "{:,}".format(
                int(stats['Number of input reads'])) + ' |'
            str_msg += "{0:.2f}".format(
                float(stats['Uniquely mapped reads %'].replace('%', ''))) + ' |'
            str_msg += "{0:.2f}".format(
                float(stats['% of reads mapped to multiple loci'].replace('%', ''))) + ' |'
            str_msg += "{:,}".format(
                int(stats['Number of splices: Annotated (sjdb)'])) + ' |'
            str_msg += "{0:.2f}".format(
                float(stats['Mismatch rate per base, %'].replace('%', ''))) + ' |'
            str_msg += '\n'
        else:
            str_msg += ' --- | --- | --- | --- |\n'
    return samples_data, str_msg


def samtools_stat_alignment_table_paired(samples_data, sample_list, alignment_path):
    """
    Print alignment report table parsing data from samtools stats
    :param samples_data: Dict returned from fastqc_trimmomatic_table with extracted data
    :param sample_list: list of samples prefix names
    :param alignment_path: Folder to alignment results
    :return: Dict with samples data and the table in string format
    """
    str_msg = '| Sample | Sorted<br>BAM | BAM<br>Index | STATS'
    str_msg += '| Mapped<br>reads | Un-Mapped<br>reads | Properly<br>paired<br>reads (%) '
    str_msg += '| Total<br>first<br>fragment<br>length '
    str_msg += '| Bases<br>mapped '
    str_msg += '| Bases<br>mapped<br>(cigar) '
    str_msg += '| Mismatches '
    str_msg += '| Error<br>rate '
    str_msg += '| Average<br>length '
    str_msg += '| Average<br>quality '
    str_msg += '|\n| --- | --- | --- | --- '
    str_msg += '| --- | --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '|\n'
    for s in sample_list:
        str_msg += '| ' + s
        str_msg += '| '
        str_msg += find_file_print_link_size(alignment_path, s, '_sorted.bam',
                                             'MB', ' --- ')
        str_msg += ' |'
        str_msg += find_file_print_link_size(alignment_path, s, '_sorted.bam.bai',
                                             'MB', ' --- ')
        str_msg += ' |'
        files = [f for ds, dr, files in os.walk(alignment_path) for f in files if
                 f.startswith(s) and f.endswith('.stats')
                 and os.path.getsize(os.path.join(alignment_path, f)) != 0]
        if len(files) == 1:
            f = os.path.relpath(os.path.join(alignment_path, files[0]))
            str_msg += get_link_size(f, 'KB', ' --- ')
            str_msg += ' |'
            stats = load_content_dict_line(f, ':', 'SN', '\t', True, 'SN\t', '')
            if s not in samples_data:
                samples_data[s] = {}
            samples_data[s]['alignment'] = {
                'mapped': int(stats['reads mapped']),
                'unmapped': int(stats['reads unmapped']),
                'properly paired': int(stats['reads properly paired'])
            }
            str_msg += "{:,}".format(int(stats['reads mapped'])) + ' |'
            str_msg += "{:,}".format(int(stats['reads unmapped'])) + ' |'
            str_msg += "{:,}".format(float(stats['percentage of properly paired reads (%)']))
            str_msg += ' |'
            str_msg += "{:,}".format(int(stats['total first fragment length'])) + ' |'
            str_msg += "{:,}".format(int(stats['bases mapped'])) + ' |'
            str_msg += "{:,}".format(int(stats['bases mapped (cigar)'].replace('%', ''))) + ' |'
            str_msg += "{:,}".format(int(stats['mismatches'].replace('%', ''))) + ' |'
            str_msg += "{:.2e}".format(float(stats['error rate'].replace('%', ''))) + ' |'
            str_msg += "{:,}".format(int(stats['average length'].replace('%', ''))) + ' |'
            str_msg += "{:.1f}".format(float(stats['average quality'].replace('%', ''))) + ' |'
            str_msg += '\n'
        else:
            str_msg += ' --- | --- | --- | --- |\n'
    return samples_data, str_msg


def samtools_stat_alignment_table_single(samples_data, sample_list, alignment_path):
    """
    Print alignment report table parsing data from samtools stats
    :param samples_data: Dict returned from fastqc_trimmomatic_table with extracted data
    :param sample_list: list of samples prefix names
    :param alignment_path: Folder to alignment results
    :return: Dict with samples data and the table in string format
    """
    str_msg = '| Sample | Sorted<br>BAM | BAM<br>Index | STATS'
    str_msg += '| Mapped<br>reads | Un-Mapped<br>reads '
    str_msg += '| Total<br>first<br>fragment<br>length '
    str_msg += '| Bases<br>mapped '
    str_msg += '| Bases<br>mapped<br>(cigar) '
    str_msg += '| Mismatches '
    str_msg += '| Error<br>rate '
    str_msg += '| Average<br>length '
    str_msg += '| Average<br>quality '
    str_msg += '|\n| --- | --- | --- | --- '
    str_msg += '| --- | --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '|\n'
    for s in sample_list:
        str_msg += '| ' + s
        str_msg += '| '
        str_msg += find_file_print_link_size(alignment_path, s, '_sorted.bam',
                                             'MB', ' --- ')
        str_msg += ' |'
        str_msg += find_file_print_link_size(alignment_path, s, '_sorted.bam.bai',
                                             'MB', ' --- ')
        str_msg += ' |'
        f = os.path.relpath(os.path.join(alignment_path, s + '.stats'))
        if os.path.exists(f) and os.path.getsize(f) != 0:
            str_msg += get_link_size(f, 'KB', ' --- ')
            str_msg += ' |'
            stats = load_content_dict_line(f, ':', 'SN', '\t', True, 'SN\t', '')
            if s not in samples_data:
                samples_data[s] = {}
            samples_data[s]['alignment'] = {
                'mapped': int(stats['reads mapped']),
                'unmapped': int(stats['reads unmapped'])
            }
            str_msg += "{:,}".format(int(stats['reads mapped'])) + ' |'
            str_msg += "{:,}".format(int(stats['reads unmapped'])) + ' |'
            str_msg += "{:,}".format(int(stats['total first fragment length'])) + ' |'
            str_msg += "{:,}".format(int(stats['bases mapped'])) + ' |'
            str_msg += "{:,}".format(int(stats['bases mapped (cigar)'].replace('%', ''))) + ' |'
            str_msg += "{:,}".format(int(stats['mismatches'].replace('%', ''))) + ' |'
            str_msg += "{:.2e}".format(float(stats['error rate'].replace('%', ''))) + ' |'
            str_msg += "{:,}".format(int(stats['average length'].replace('%', ''))) + ' |'
            str_msg += "{:.1f}".format(float(stats['average quality'].replace('%', ''))) + ' |'
            str_msg += '\n'
        else:
            str_msg += ' --- | --- | --- | --- |\n'
    return samples_data, str_msg


def alignment_table(samples_data, sample_list, alignment_path, alignment_tool):
    """
Print alignment report table parsing data from STAR
    :param samples_data: Dict returned from fastqc_trimmomatic_table with extracted data
    :param sample_list: list of samples prefix names
    :param alignment_path: Folder to alignment results
    :param alignment_tool: Tools used for the alignment
    :return:Dict with samples data and the table in string format
    """
    if alignment_tool == 'STAR_single':
        return star_alignment_table_single(samples_data, sample_list, alignment_path)
    elif alignment_tool == 'BWA_paired' or alignment_tool == 'STAR_paired':
        return samtools_stat_alignment_table_paired(samples_data, sample_list, alignment_path)
    elif alignment_tool == 'BWA_single':
        return samtools_stat_alignment_table_single(samples_data, sample_list, alignment_path)


def star_reads_distribution_plot_single(samples_data, sample_list, fig_size):
    """
    Create the distribution of reads plot for single-end data
    :param samples_data: Dict returned from alignment_table with extracted data
    :param sample_list: list of samples prefix names
    :param fig_size: Tuple with figure size
    :return: matplotlib plot
    """
    f, plot = plt.subplots(figsize=fig_size)
    N = 0
    unique = []
    multiple = []
    unmapped = []
    trimmed = []
    for s in sample_list:
        if 'alignment' in samples_data[s]:
            N += 1
            unique.append(samples_data[s]['alignment']['unique'])
            multiple.append(samples_data[s]['alignment']['multiple'])
            unmapped.append(samples_data[s]['alignment']['unmapped'])
            trimmed.append(
                samples_data[s]['fastqc']['tot_seq'] - samples_data[s]['trimmed']['tot_seq'])
    ind = np.arange(N)
    width = 0.45

    p4 = plot.bar(ind, trimmed, width, color='#2eea15')
    p3 = plot.bar(ind, unmapped, width,
                  bottom=np.array(trimmed), color='#ea1563')
    p2 = plot.bar(ind, multiple, width,
                  bottom=np.array(trimmed) + np.array(unmapped), color='#ff7311')
    p1 = plot.bar(ind, unique, width,
                  bottom=np.array(trimmed) + np.array(unmapped) + np.array(multiple),
                  color='#1c6cab')

    plt.ylabel('No. of Reads')
    plt.xticks(ind, sample_list, rotation='vertical')
    plt.legend((p1[0], p2[0], p3[0], p4[0]),
               ('Unique Mapped', 'Multiple Mapped', 'Unmapped', 'Trimmed'))

    return plot


def star_reads_distribution_plot_paired(samples_data, sample_list, fig_size):
    """
    Create the distribution of reads plot for paired-end data
    :param samples_data: Dict returned from alignment_table with extracted data
    :param sample_list: list of samples prefix names
    :param fig_size: Tuple with figure size
    :return: matplotlib plot
    """
    f, plot = plt.subplots(figsize=fig_size)
    N = 0
    mapped = []
    unmapped = []
    for s in sample_list:
        if s in samples_data:
            if 'alignment' in samples_data[s]:
                N += 1
                mapped.append(samples_data[s]['alignment']['properly paired'])
                unmapped.append(samples_data[s]['alignment']['unmapped'])
    ind = np.arange(N)
    width = 0.50

    p2 = plot.bar(ind, unmapped, width, color='#ea1563')
    p1 = plot.bar(ind, mapped, width, bottom=np.array(unmapped), color='#1c6cab')

    plt.ylabel('No. of Reads')
    plt.xticks(ind, sample_list, rotation='vertical')
    plt.legend((p1[0], p2[0]), ('Properly paired', 'Unmapped'))

    return plot


def samtools_reads_distribution_plot_single(samples_data, sample_list, fig_size):
    """
    Create the distribution of reads plot from Samtools stats
    :param samples_data: Dict returned from alignment_table with extracted data
    :param sample_list: list of samples prefix names
    :param fig_size: Tuple with figure size
    :return: matplotlib plot
    """
    f, plot = plt.subplots(figsize=fig_size)
    N = 0
    mapped = []
    unmapped = []
    trimmed = []
    for s in sample_list:
        if 'alignment' in samples_data[s]:
            N += 1
            mapped.append(samples_data[s]['alignment']['mapped'])
            unmapped.append(samples_data[s]['trimmed']['tot_seq'] -
                            samples_data[s]['alignment']['mapped'])
            trimmed.append(samples_data[s]['fastqc']['tot_seq'] -
                           samples_data[s]['trimmed']['tot_seq'])
    ind = np.arange(N)
    width = 0.45

    p3 = plot.bar(ind, trimmed, width, color='#2eea15')
    p2 = plot.bar(ind, unmapped, width,
                  bottom=np.array(trimmed), color='#ea1563')
    p1 = plot.bar(ind, mapped, width,
                  bottom=np.array(trimmed) + np.array(unmapped), color='#1c6cab')

    plt.ylabel('No. of Reads')
    plt.xticks(ind, sample_list, rotation='vertical')
    plt.legend((p1[0], p2[0], p3[0]),
               ('Mapped', 'Unmapped', 'Trimmed'))

    return plot


def samtools_reads_distribution_plot_paired(samples_data, sample_list, fig_size):
    """
    Create the distribution of reads plot from Samtools stats
    :param samples_data: Dict returned from alignment_table with extracted data
    :param sample_list: list of samples prefix names
    :param fig_size: Tuple with figure size
    :return: matplotlib plot
    """
    f, plot = plt.subplots(figsize=fig_size)
    N = 0
    mapped = []
    ppaired = []
    unmapped = []
    for s in sample_list:
        if 'alignment' in samples_data[s] and 'properly paired' in samples_data[s]['alignment']:
            N += 1
            mapped.append(samples_data[s]['alignment']['mapped'])
            unmapped.append(samples_data[s]['alignment']['unmapped'])
            ppaired.append(samples_data[s]['alignment']['properly paired'])
    ind = np.arange(N)
    width = 0.45

    p3 = plot.bar(ind, unmapped, width, color='#2eea15')
    p2 = plot.bar(ind, ppaired, width,
                  bottom=np.array(unmapped), color='#ea1563')
    p1 = plot.bar(ind, mapped, width,
                  bottom=np.array(unmapped) + np.array(ppaired), color='#1c6cab')

    plt.ylabel('No. of Reads')
    plt.xticks(ind, sample_list, rotation='vertical')
    plt.legend((p1[0], p2[0], p3[0]),
               ('Mapped', 'Properly paired', 'Unmapped'))

    return plot


def reads_distribution_plot(samples_data, sample_list, fig_size, alignment_tool):
    """
    Create the distribution of reads plot
    :param samples_data: Dict returned from alignment_table with extracted data
    :param sample_list: list of samples prefix names
    :param fig_size: Tuple with figure size
    :param alignment_tool: Tools used for the alignment
    :return: matplotlib plot
    """
    if alignment_tool == 'STAR_single':
        return star_reads_distribution_plot_single(samples_data, sample_list, fig_size)
    elif alignment_tool == 'STAR_paired':
        return star_reads_distribution_plot_paired(samples_data, sample_list, fig_size)
    elif alignment_tool == 'BWA_single':
        return samtools_reads_distribution_plot_single(samples_data, sample_list, fig_size)
    elif alignment_tool == 'BWA_paired':
        return samtools_reads_distribution_plot_paired(samples_data, sample_list, fig_size)



import os

from pm4ngs.jupyterngsplugin.files.fastq.fastqc import parse_fastqc_zip
from pm4ngs.jupyterngsplugin.markdown.utils import find_file_print_link_size


def fastqc_table(sample_list, samples_path, fastqc_path):
    """
    Print FastQC report table parsing data
    :param sample_list: list of samples prefix names
    :param samples_path: Path to original fastq files
    :param fastqc_path: Path to FastQC output folder
    :return: Dict with samples data and the table in string format
    """
    samples_data = {}
    str_msg = '| Sample | Fastq | FastQC<br>Report | No of Reads<br>in fastq | Seq<br> Len | %GC '
    str_msg += '| Poor<br>Quality | Fail<br>Tests |\n'
    str_msg += '| --- | --- | --- |--- | --- | --- | --- | --- |\n'
    for s in sample_list:
        str_msg += '| ' + s
        str_msg += '| '
        str_msg += find_file_print_link_size(samples_path, s, '.fastq.gz', 'MB', ' --- ')
        str_msg += ' |'
        str_msg += find_file_print_link_size(fastqc_path, s, '.html', 'MB', ' --- ')
        str_msg += ' |'
        f = os.path.relpath(os.path.join(fastqc_path, s + '_fastqc.zip'))
        if os.path.exists(f) and os.path.getsize(f) != 0:
            tests, tot_seq, poor_quality, seq_len, gc_content = parse_fastqc_zip(f)
            samples_data[s] = {'fastqc': {
                'tests': tests,
                'tot_seq': tot_seq,
                'poor_quality': poor_quality,
                'seq_len': seq_len,
                'gc_content': gc_content
            }
            }
            str_msg += "{:,}".format(tot_seq) + '|'
            str_msg += seq_len + '|'
            str_msg += gc_content + '|'
            str_msg += str(poor_quality) + '|'
            fail_tests = ''
            for t in tests:
                if tests[t] == 'FAIL':
                    if fail_tests:
                        fail_tests += '<br>'
                    fail_tests += t
            str_msg += fail_tests + '|\n'

        else:
            str_msg += ' --- | --- | --- | --- | --- |\n'
    return samples_data, str_msg


def fastqc_trimmomatic_table(samples_data, sample_list, trimmomatic_path):
    """
    Print FastQC report table parsing data for trimmed samples
    :param samples_data: Dict returned from fastqc_table with extracted data
    :param sample_list: list of samples prefix names
    :param trimmomatic_path: Folder with trimmed samples and FastQC reports
    :return: Dict with samples data and the table in string format
    """
    str_msg = '| Sample | Trimmed<br>Fastq | FastQC<br>Report | No of Reads<br>in fastq |'
    str_msg += 'Removed<br>Reads | Seq<br> Len | %GC | Poor<br>Quality | Fail<br>Tests |\n'
    str_msg += '| --- | --- | --- | --- | --- | --- | --- | --- | --- |\n'
    for s in sample_list:
        str_msg += '| ' + s
        str_msg += '| '
        str_msg += find_file_print_link_size(trimmomatic_path, s, '.fastq.gz', 'MB', ' --- ')
        str_msg += ' |'
        str_msg += find_file_print_link_size(trimmomatic_path, s, '.html', 'KB', ' --- ')
        str_msg += ' |'
        files = [f for ds, dr, files in os.walk(trimmomatic_path) for f in files if
                 f.startswith(s) and f.endswith('_fastqc.zip')
                 and os.path.getsize(os.path.join(trimmomatic_path, f)) != 0]
        if len(files) == 1:
            f = os.path.relpath(os.path.join(trimmomatic_path, files[0]))
            tests, tot_seq, poor_quality, seq_len, gc_content = parse_fastqc_zip(f)
            str_msg += "{:,}".format(tot_seq) + '|'
            str_msg += "{:,}".format(samples_data[s]['fastqc']['tot_seq'] - tot_seq) + '|'
            str_msg += seq_len + '|'
            str_msg += gc_content + '|'
            str_msg += str(poor_quality) + '|'
            fail_tests = ''
            for t in tests:
                if tests[t] == 'FAIL':
                    if fail_tests:
                        fail_tests += '<br>'
                    fail_tests += t
            str_msg += fail_tests + '|\n'
            samples_data[s]['trimmed'] = {
                'tests': tests,
                'tot_seq': tot_seq,
                'poor_quality': poor_quality,
                'seq_len': seq_len,
                'gc_content': gc_content
            }
        else:
            str_msg += ' --- | --- | --- | --- | --- | --- |\n'
    return samples_data, str_msg



#!/usr/bin/env python
import argparse
import os
import subprocess

import requests

from pm4ngs import __version__
from pm4ngs.cookiecutter import execute_cookiecutter

PIPELINES = {
    1: {
        'name': 'RNA-Seq',
        'url': 'https://github.com/ncbi/pm4ngs-rnaseq',
        'example_yml':
            'https://raw.githubusercontent.com/ncbi/pm4ngs-rnaseq/master'
            '/example/pm4ngs_rnaseq_demo_config.yaml',
        'sample_sheet':
            'https://raw.githubusercontent.com/ncbi/pm4ngs-rnaseq/master'
            '/example/pm4ngs_rnaseq_demo_sample_data.csv'
    },
    2: {
        'name': 'ChIP-Seq',
        'url': 'https://github.com/ncbi/pm4ngs-chipseq',
        'example_yml':
            'https://raw.githubusercontent.com/ncbi/pm4ngs-chipseq/master'
            '/example/pm4ngs_chipseq_demo_config.yaml',
        'sample_sheet':
            'https://raw.githubusercontent.com/ncbi/pm4ngs-chipseq/master'
            '/example/pm4ngs_chipseq_demo_sample_data.csv'
    },
    3: {
        'name': 'ChIP-exo',
        'url': 'https://github.com/ncbi/pm4ngs-chipexo',
        'example_yml':
            'https://raw.githubusercontent.com/ncbi/pm4ngs-chipexo/master'
            '/example/pm4ngs_chipexo_demo_config.yaml',
        'sample_sheet':
            'https://raw.githubusercontent.com/ncbi/pm4ngs-chipexo/master'
            '/example/pm4ngs_chipexo_demo_sample_data.csv'
    },
}


def command_line(pipeline, is_general=False):
    if pipeline > 0:
        description = 'Generate a PM4NGS project for {} data analysis'.format(
            PIPELINES[pipeline]['name'])
    else:
        description = 'Generate a PM4NGS project from cookiecutter template'
    parser = argparse.ArgumentParser(description)

    parser.add_argument('-v', '--version', action='version',
                        version='PM4NGS version: {}'.format(__version__))
    parser.add_argument('--sample-sheet',
                        help='Sample sheet CSV file with columns: '
                             'sample_name,file,condition,replicate',
                        required=True)
    parser.add_argument('--config-file',
                        help='User configuration file with project description',
                        required=False)
    parser.add_argument('--copy-rawdata',
                        help='Copy the raw data defined in the sample table '
                             'to the project/data/DATASET folder',
                        action='store_true',
                        required=False)

    if is_general:
        parser.add_argument('--template',
                            help='PM4NGS cookiecutter template',
                            required=True)
    args = parser.parse_args()
    return args


def rnaseq():
    create_predefined(1)


def rnaseq_demo():
    generate_demo(1)


def chipseq():
    create_predefined(2)


def chipseq_demo():
    generate_demo(2)


def chipexo():
    create_predefined(3)


def chipexo_demo():
    generate_demo(3)


def create_predefined(pipeline):
    args = command_line(pipeline)
    print('Generating {} data analysis project'.format(
        PIPELINES[pipeline]['name']))
    execute_cookiecutter(PIPELINES[pipeline]['url'], args.config_file,
                         args.sample_sheet, args.copy_rawdata)


def generate_demo(pipeline):
    print('Generating demo for {} data analysis project'.format(
        PIPELINES[pipeline]['name']))
    config_file = os.path.basename(PIPELINES[pipeline]['example_yml'])
    print('Downloading file: ' + config_file)
    r = requests.get(PIPELINES[pipeline]['example_yml'], allow_redirects=True)
    open(config_file, 'wb').write(r.content)
    sample_sheet = os.path.basename(PIPELINES[pipeline]['sample_sheet'])
    print('Downloading file: ' + sample_sheet)
    r = requests.get(PIPELINES[pipeline]['sample_sheet'], allow_redirects=True)
    open(sample_sheet, 'wb').write(r.content)
    execute_cookiecutter(PIPELINES[pipeline]['url'], config_file,
                         sample_sheet, False)


def create_project():
    args = command_line(0, True)
    print('Generating data analysis project from template: {}'.format(
        args.template))
    execute_cookiecutter(args.template, args.config_file,
                         args.sample_sheet, args.copy_rawdata)


def start_server_main():
    parser = argparse.ArgumentParser(
        description='Start the PM4NGS jupyter server')

    parser.add_argument('-v', '--version', action='version',
                        version='PM4NGS version: {}'.format(__version__))
    parser.add_argument('--no_browser',
                        help='Don\'t open the notebook in a browser after startup',
                        action='store_true',
                        required=False)
    parser.add_argument('--ip',
                        help='The IP address the notebook server will listen on. '
                             'Default: localhost',
                        type=str,
                        required=False)
    parser.add_argument('--port',
                        help='The port the notebook server will listen on. '
                             'Default: 8888',
                        type=int,
                        required=False)

    args = parser.parse_args()
    jupyter_cmd = "jupyter notebook"
    if args.no_browser:
        jupyter_cmd += " --NotebookApp.open_browser=False"
    else:
        jupyter_cmd += " --NotebookApp.open_browser=True"
    if args.ip:
        jupyter_cmd += " --ip=" + args.ip
    if args.port:
        jupyter_cmd += " --port={}".format(args.port)
    print('Starting server with command:\n{}'.format(jupyter_cmd))
    subprocess.run(jupyter_cmd, shell=False)



from PIL import Image
from pm4ngs.jupyterngsplugin.image.pdftobase64 import piltobase64


def imagetobase64(png_file, width, height):
    """
    Return base64 image string image from a PDF file to be used in HTML code
    :param png_file_name: PNG file name
    :param width: Final image width
    :param height: Final image height
    :return: base64 image string
    """
    pil_image = Image.open(png_file)
    pil_image = pil_image.resize((width, height), Image.ANTIALIAS)
    return piltobase64(pil_image)



import os
from pm4ngs.jupyterngsplugin.markdown.utils import get_link_image


def print_table_cell(f):
    str_msg = ''
    with open(f) as fin:
        for line in fin:
            fields = line.strip().split('\t')
            str_msg += "{:.2e}".format(float(fields[1])) + ' |'

            for i in fields[2].split(','):
                str_msg += "{:,}".format(int(i)) + '<br>'
            str_msg += ' |'

            for i in fields[3].split(','):
                str_msg += "{:.2f}".format(float(i)) + '<br>'
            str_msg += ' |'

            str_msg += fields[4] + ' |'
            str_msg += "{:.2f}".format(float(fields[5])) + ' |'

            str_msg += fields[6] + ' |'

            str_msg += "{:.2f}".format(float(fields[7])) + ' |'
            if float(fields[8]) < 1.05:
                str_msg += '<font color=\'red\'>'
            str_msg += "{:.2f}".format(float(fields[8]))
            if float(fields[8]) < 1.05:
                str_msg += '</font>'
            str_msg += ' |'
            if float(fields[9]) < 0.8:
                str_msg += '<font color=\'red\'>'
            str_msg += "{:.2f}".format(float(fields[9]))
            if float(fields[9]) < 0.8:
                str_msg += '</font>'
            str_msg += ' |'
            try:
                str_msg += "{:.2f}".format(float(fields[10])) + ' |'
            except ValueError:
                str_msg += ' --- |'
    return str_msg


def qc_table(sample_list, alignment_path, width, height):
    str_msg = '| Sample '
    str_msg += '| numReads '
    str_msg += '| estFragLen '
    str_msg += '| corr_estFragLen '
    str_msg += '| phantomPeak '
    str_msg += '| corr_phantomPeak '
    str_msg += '| argmin_corr '
    str_msg += '| min_corr '
    str_msg += '| NSC '
    str_msg += '| RSC '
    str_msg += '| QualityTag '
    str_msg += '| strand-shift cc '
    str_msg += '|\n| --- | --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '|\n'
    for s in sample_list:
        str_msg += '| ' + s
        str_msg += '| '
        f = os.path.relpath(os.path.join(alignment_path, s + '_sorted.tagAlign.cc.qc'))
        if os.path.exists(f) and os.path.getsize(f) != 0:
            str_msg += print_table_cell(f)
        else:
            str_msg += ' --- | --- | --- | --- | --- | --- | --- | --- | --- | --- |'
        f = os.path.relpath(os.path.join(alignment_path, s + '_sorted.tagAlign.cc.plot.pdf'))
        if os.path.exists(f) and os.path.getsize(f) != 0:
            str_msg += get_link_image(f, width, height, ' --- |\n')
        else:
            str_msg += ' --- |'
        str_msg += '\n'
    str_msg += '\n'
    return str_msg



import os

import pandas

from pm4ngs.jupyterngsplugin.markdown.utils import get_link_image
from pm4ngs.jupyterngsplugin.markdown.utils import get_link_text
from pm4ngs.jupyterngsplugin.utils.count_lines import count_lines


def peak_calling_table_with_qc(factors, alignment_path, peak_calling_path, width, height):
    """
    Create a peck calling table with phantompeakqualtools plots
    :param factors: Pandas DataFrame with the samples and conditions
    :param alignment_path: Path to the alignments results
    :param peak_calling_path: Path to the peak calling results
    :param width: Image width
    :param height: Image height
    :return: An string in markdown language
    """
    conditions = factors['condition'].unique()
    n_max_samples = 0
    for c in conditions:
        rep = factors[factors['condition'] == c]['sample_name']
        if len(rep) > n_max_samples:
            n_max_samples = len(rep)

    str_msg = '| Condition '
    str_msg += '| No. Peaks '
    for i in range(0, n_max_samples):
        str_msg += '| Sample {0} '.format(i)
    str_msg += '|\n'
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '|\n'

    for c in conditions:
        str_msg += '| ' + c
        str_msg += '| '
        r0 = os.path.relpath(os.path.join(peak_calling_path, c + '.border_pair_annot.bed'))
        if os.path.exists(r0) and os.path.getsize(r0) != 0:
            str_msg += get_link_text(r0, count_lines(r0, '#'), ' --- ')
            str_msg += '|'
        else:
            str_msg += ' --- |'
        rep = factors[factors['condition'] == c]['sample_name']
        for r in rep:
            f = os.path.relpath(os.path.join(alignment_path, r + '_sorted.tagAlign.cc.plot.pdf'))
            str_msg += get_link_image(f, width, height, ' --- ')
            str_msg += '|'
        if len(rep) < n_max_samples:
            for i in range(len(rep), n_max_samples):
                str_msg += ' --- |'
        str_msg += '\n'
    str_msg += '\n'
    str_msg += '\nClick on figure to retrieve original PDF file\n\n'

    return str_msg


def meme_motif_table_condition(motif_path, c, d, width, height):
    str_msg = ''
    f = os.path.relpath(os.path.join(motif_path, c + '.border_pair_annot_' + d,
                                     'summary.tsv'))
    if os.path.exists(f) and os.path.getsize(f) != 0:
        df = pandas.read_csv(f, sep='\t', comment='#')
        if not df.empty:
            n_motif = len(df)
            source = df.iloc[0]['MOTIF_SOURCE']
            consensus = df.iloc[0]['CONSENSUS']
            w = df.iloc[0]['WIDTH']
            sites = df.iloc[0]['SITES']
            e_value = df.iloc[0]['E-VALUE']
            lhtml = os.path.relpath(os.path.join(motif_path, c + '.border_pair_annot_' + d,
                                                 'meme-chip.html'))
            str_msg += '| ' + get_link_text(lhtml, c, ' --- ')
            str_msg += '|'

            str_msg += " {} | {} | {} | {} | {} | {} | ".format(n_motif, source, consensus,
                                                                w, sites, e_value)
            if source == 'MEME':
                lpng = os.path.relpath(
                    os.path.join(motif_path, c + '.border_pair_annot_' + d, 'meme_out',
                                 'logo1.png'))
                if os.path.exists(lpng) and os.path.getsize(lpng) != 0:
                    str_msg += get_link_image(lpng, width, height, ' --- ')
                    str_msg += ' |'
                else:
                    str_msg += ' --- |'
            elif source == 'DREME':
                lout = os.path.relpath(os.path.join(motif_path, c
                                                    + '.border_pair_annot_'
                                                    + d,
                                                    'dreme_out'))
                dreme_files = [f for ds, dr, files in os.walk(lout)
                               for f in files if f.startswith('m01nc')]
                for dm in dreme_files:
                    loutfile = os.path.relpath(
                        os.path.join(motif_path, c + '.border_pair_annot_' + d,
                                     'dreme_out', dm))
                    if os.path.exists(loutfile) and os.path.getsize(loutfile) != 0:
                        str_msg += get_link_image(loutfile, width, height, ' --- ')
                        str_msg += ' |'
                    else:
                        str_msg += ' --- |'
            str_msg += '\n'
    return str_msg


def meme_motif_table(factors, motif_path, width, height):
    """
    Create a table for the MEME motif find results
    :param factors: Pandas DataFrame with the samples and conditions
    :param motif_path: Path to the MEME results
    :param width: Image width
    :param height: Image height
    :return: An string in markdown language
    """
    conditions = factors['condition'].unique()
    memedbs = []
    dirs = [d for d in os.listdir(motif_path) if os.path.isdir(os.path.join(motif_path, d))]
    for d in dirs:
        if '.border_pair_annot_' in d:
            d = d.split('.border_pair_annot_')[1]
            memedbs.append(d)
    memedbs = list(set(memedbs))
    str_msg = ""
    for d in memedbs:
        str_msg += "### Database: {}\n\n".format(d)
        str_msg += "| Sample/<br>MEME output | No.<br>Motifs | 1st motif<br>source "
        str_msg += "| 1st motif<br>consensus | 1st motif<br>width | 1st motif<br>sites "
        str_msg += "| 1st motif<br>E-Value | 1st motif |\n"
        str_msg += "| --- | --- | --- | --- | --- | --- | --- | --- |\n"
        for c in conditions:
            str_msg += meme_motif_table_condition(motif_path, c, d, width, height)
        str_msg += '\n\n\n'
    return str_msg



import os
import pandas
import seaborn as sns
import matplotlib.pyplot as plt


def tpmcalculator_distribution_plot(column, output_folder, output_suffix, fig_size):
    """
    Parse a TPMCalculator output and create distribution plot
    :param columns: TPMCalculator output column to plot
    :param output_folder: TPMCalculator output folder
    :param output_folder: TPMCalculator output file name sufix
    :param fig_size:  Tuple with figure size
    :return: matplotlib plot
    """
    files = [f for ds, df, files in os.walk(output_folder) for f in files if output_suffix in f]
    matrix_file = os.path.join(output_folder, column + '.tsv')
    if os.path.exists(matrix_file) and os.path.getsize(matrix_file) != 0:
        data = pandas.read_csv(matrix_file, sep='\t')
        f, plot = plt.subplots(figsize=fig_size)
        toPlot = []
        for f in files:
            if f.endswith('.gz'):
                output_suffix_real = output_suffix + '.gz'
            else:
                output_suffix_real = output_suffix
            s = f.replace(output_suffix_real, '')
            if s in data:
                for r in data[s]:
                    toPlot.append([r, s])
        d = pandas.DataFrame(toPlot, columns=[column, 'Sample'])
        sns.boxplot(y='Sample', x=column, data=d, orient="h", palette="Set2")



import os


def check_errors_from_logs(path, log_suffix):
    """
    Check all files with name finishing on the `log_suffix` variable.
    If file doesn't ends with the line:  'Final process status is success'
    the file is reported with error
    :param path: Path to log files
    :param log_suffix: Siffix of logs files to process
    """
    logs = [f for ds, dr, files in os.walk(path) for f in files if f.endswith(log_suffix)]
    if len(logs) == 0:
        print('No logs in folder: ' + path + 'with suffix: ' + log_suffix)
    else:
        failed = False
        for log in logs:
            log_file = os.path.join(path, log)
            with open(log_file, 'r') as fin:
                for line in fin:
                    pass
                if 'Final process status is success' not in line.strip():
                    print('Process no completed or failed.\n'
                          'Please, do not proceed to next cells\n'
                          'Check log file: ' + log_file)
                    failed = True
        if not failed:
            print('Process completed.\nProceed to next cells')
        else:
            print('There are errors in: ' + str(failed) + '/' + str(len(logs)))


def check_cwl_command_log(log_file):
    """
    Check all files with name finishing on the `log_suffix` variable.
    If file doesn't ends with the line:  'Final process status is success'
    the file is reported with error
    :param fastqc.log: log file
    """
    with open(log_file, 'r') as fin:
        failed = False
        for line in fin:
            if 'completed permanentFail' in line:
                print('Process failed.\n'
                      'Please, do not proceed to next cells\n'
                      'Check log file: ' + log_file)
                failed = True
                break
        if not failed:
            if 'Final process status is success' in line.strip():
                print('Process completed.\nProceed to next cells')
            else:
                print('Process no completed.\n'
                      'Please, do not proceed to next cells\n'
                      'Check log file: ' + log_file)



import os
import pandas

from pm4ngs.jupyterngsplugin.image.pngtobase64 import imagetobase64


def go_plots_table(conditions, data_path, width, height):
    str_msg = ''
    for c in conditions:
        str_msg += '\n\n### Condition: ' + c.replace('_vs_', ' vs ') + '\n\n'

        str_msg += '\n\n| Biological process | Molecular Function | Cellular Component |\n'
        str_msg += '| --- | --- | --- |\n'
        f = os.path.relpath(os.path.join(data_path, 'go_biological_process_' + c + '_union.png'))
        if os.path.exists(f) and os.path.getsize(f) != 0:
            str_msg += '| <a href="'
            str_msg += f.replace(' ', '%20')
            str_msg += '" target="_blank">'
            str_msg += '<img src="data:image/png;base64,' + imagetobase64(f, width, height) + '">'
            str_msg += '</a> |'
        else:
            str_msg += '| --- |'
        f = os.path.relpath(os.path.join(data_path, 'go_molecular_function_' + c + '_union.png'))
        if os.path.exists(f) and os.path.getsize(f) != 0:
            str_msg += ' <a href="'
            str_msg += f.replace(' ', '%20')
            str_msg += '" target="_blank">'
            str_msg += '<img src="data:image/png;base64,' + imagetobase64(f, width, height) + '">'
            str_msg += '</a> |'
        else:
            str_msg += ' --- |'
        f = os.path.relpath(os.path.join(data_path, 'go_cellular_component_' + c + '_union.png'))
        if os.path.exists(f) and os.path.getsize(f) != 0:
            str_msg += ' <a href="'
            str_msg += f.replace(' ', '%20')
            str_msg += '" target="_blank">'
            str_msg += '<img src="data:image/png;base64,' + imagetobase64(f, width, height) + '">'
            str_msg += '</a> |'
        else:
            str_msg += ' --- |\n'
    str_msg += '\n\n'
    return str_msg


def print_count_table(c, namespaces, data_path):
    str_msg = ''
    for n in namespaces:
        str_msg += '|' + n.replace('_', ' ') + '|'
        f = os.path.relpath(os.path.join(data_path, 'go_over_' + n + '_' + c + '_union.csv'))
        if os.path.exists(f) and os.path.getsize(f) != 0:
            df = pandas.read_csv(f)
            str_msg += ' {0} |'.format(len(df))
        else:
            str_msg += '--- |'
        f = os.path.relpath(os.path.join(data_path, 'go_under_' + n + '_' + c + '_union.csv'))
        if os.path.exists(f) and os.path.getsize(f) != 0:
            df = pandas.read_csv(f)
            str_msg += ' {0} |'.format(len(df))
        else:
            str_msg += '--- |'
        str_msg += '\n'
    str_msg += '\n\n'
    return str_msg


def print_table_cell(filename, title):
    str_msg = ''
    table_header = '''
    <table>
    <thead>
    <tr>
    <th>GO term</th>
    <th>Name</th>
    <th>FDR</th>
    </tr>
    </thead>
    <tbody>
    '''
    if os.path.exists(filename) and os.path.getsize(filename) != 0:
        df = pandas.read_csv(filename)
        if len(df) > 0:
            str_msg += '<div style="display: table-cell;vertical-align:top;">'
            str_msg += '<h4>'
            str_msg += title
            str_msg += '</h4>'
            str_msg += '<br>CSV file:<br><a href="'
            str_msg += filename.replace(' ', '%20')
            str_msg += '" target="_blank">'
            str_msg += 'raw file'
            str_msg += '</a><br>'
            str_msg += table_header
            for i, r in df.head(30).iterrows():
                str_msg += '<tr>'
                str_msg += '<td>'
                str_msg += '<a href="https://www.ebi.ac.uk/QuickGO/term/' + r['term']
                str_msg += '" target="_blank">'
                str_msg += r['term'] + '</a>'
                str_msg += '</td>'
                str_msg += '<td>' + r['name'][:40] + '</td>'
                str_msg += '<td>' + "{:.3e}".format(r['q']) + '</td>'
                str_msg += '</tr>'
            str_msg += '</tbody></table>'
            str_msg += '</div>'
    return str_msg


def go_html_table(conditions, data_path):
    str_msg = ''
    for c in conditions:
        str_msg += '\n\n### Condition: ' + c.replace('_vs_', ' vs ') + '\n\n'
        str_msg += '#### GO terms per name space \n\n'
        namespaces = [f.replace('go_over_', '').replace('_' + c + '_union.csv', '')
                      for d, ds, files in
                      os.walk(data_path) for f in files
                      if f.startswith('go_over_') and f.endswith('.csv') and c in f]
        str_msg += '| Name space | Over | Under |\n'
        str_msg += '| --- | --- | --- |\n'
        str_msg += print_count_table(c, namespaces, data_path)
        str_msg += '\n\n'

        for n in namespaces:
            str_msg += '<div><h3>Top 30 GO terms for ' + n.replace('_', ' ') + '</h3>'
            str_msg += '<div style="display: table;width: 100%;">'
            str_msg += '<div style="display: table-row;">'
            f = os.path.relpath(os.path.join(data_path, 'go_over_' + n + '_' + c + '_union.csv'))
            str_msg += print_table_cell(f, 'Over-expressed')
            f = os.path.relpath(os.path.join(data_path, 'go_under_' + n + '_' + c + '_union.csv'))
            str_msg += print_table_cell(f, 'Under-expressed')
            str_msg += '</div></div></div>'
        str_msg += '\n'
    return str_msg



import requests
from pm4ngs.main import PIPELINES


def test_main_rnase_name():
    assert PIPELINES[1]['name'] == 'RNA-Seq'


def test_main_rnase_url():
    url = PIPELINES[1]['url']
    request = requests.get(url)
    assert request.status_code == 200



import yaml


def write_to_yaml(src, file_name):
    with open(file_name, 'w') as file:
        yaml.dump(src, file)


def load_from_yaml(file_name):
    data = None
    with open(file_name) as f:
        data = yaml.load(f, Loader=yaml.FullLoader)
    return data



import xmltodict
import collections
import pandas
from Bio import Entrez

from time import sleep
from urllib.error import HTTPError


def runs_from_bioproject_id(bioproject_id, email, eutils_key=None):
    Entrez.email = email
    if eutils_key:
        Entrez.api_key = eutils_key
    handle = Entrez.esearch(db="sra", term=bioproject_id, retmax=50)
    sra_id = Entrez.read(handle)
    handle.close()
    tuples = []
    i = len(sra_id['IdList']) - 1
    while i >= 0:
        try:
            handle = Entrez.efetch(db="sra",
                                   id=sra_id['IdList'][i],
                                   rettype="full", retmode="full")
            sra = xmltodict.parse(handle.read())
            handle.close()
            sra_data = sra['EXPERIMENT_PACKAGE_SET']['EXPERIMENT_PACKAGE']
            alias = '' if 'SAMPLE' not in sra_data else sra_data['SAMPLE']['@alias']
            accession = '' if 'SAMPLE' not in sra_data else sra_data['SAMPLE']['@accession']
            if type(sra_data['RUN_SET']['RUN']) is list:
                for run in sra_data['RUN_SET']['RUN']:
                    f = run['@accession']
                    tuples.append([alias, accession, f])
            elif type(sra_data['RUN_SET']['RUN']) is collections.OrderedDict:
                f = sra_data['RUN_SET']['RUN']['@accession']
                tuples.append([alias, accession, f])
            i -= 1
        except HTTPError as err:
            if err.code == 429:
                print('SRA metadata downloaded for %d/%d. '
                      'Max request reached. Retry-After: %s sec' %
                      (len(sra_id['IdList']) - i,
                       len(sra_id['IdList']),
                       err.headers['Retry-After']))
                sleep(int(err.headers['Retry-After']))
                continue

    return pandas.DataFrame(tuples, columns=['ALIAS', 'ACCESSION', 'RUN'])



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html


import os
import sys
# sys.path.insert(0, os.path.abspath('.'))

import mock
MOCK_MODULES = ['yaml', 'fuzzywuzzy']

for mod_name in MOCK_MODULES:
    sys.modules[mod_name] = mock.Mock()

sys.path.append(os.path.abspath(__file__) + "../shared/")


# -- Project information -----------------------------------------------------

project = 'pm4ngs'
copyright = '2020, Roberto Vera Alvarez'
author = 'Roberto Vera Alvarez'

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinxarg.ext',
    'sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx.ext.intersphinx',
    'sphinx.ext.todo',
    'sphinx.ext.coverage',
    'sphinx.ext.mathjax',
    'sphinx.ext.ifconfig',
    'sphinx.ext.viewcode',
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']
source_suffix = '.rst'
master_doc = 'index'

exec(open('../../src/pm4ngs/__init__.py').read())

# The short X.Y version.
version = __version__
# The full version, including alpha/beta/rc tags.
release = __version__

# An rst epilog to apper at the end of every page
rst_epilog = """
"""

language = 'python'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ['_build']
pygments_style = 'sphinx'

# -- Options for HTML output -------------------------------------------------

on_rtd = os.environ.get('READTHEDOCS', None) == 'True'
if not on_rtd:  # only import and set the theme if we're building docs locally
    import sphinx_rtd_theme
    html_theme = 'sphinx_rtd_theme'  # 'alabaster' 'sphinx_rtd_theme'
    html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']
html_context = {
    'css_files': [
        '_static/theme_overrides.css',  # override wide tables in RTD theme
        ],
     }

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
html_use_smartypants = False
smartquotes = False

# Output file base name for HTML help builder.
htmlhelp_basename = 'pm4ngsdoc'

# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    # 'preamble': '',

    # Latex figure (float) alignment
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [(master_doc, 'pm4ngs.tex', u'PM4NGS Documentation', u'Roberto Vera Alvarez', 'manual'), ]

# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [(master_doc, 'snakepipes', u'snakePipes Documentation', [author], 1)]

# If true, show URL addresses after external links.
# man_show_urls = False


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'PM4NGS', u'PM4NGS Documentation', author, 'PM4NGS', 'One line description of project.', 'Miscellaneous'), ]

# Documents to append as an appendix to all manuals.
# texinfo_appendices = []

# If false, no module index is generated.
# texinfo_domain_indices = True

# How to display URL addresses: 'footnote', 'no', or 'inline'.
# texinfo_show_urls = 'footnote'

# If true, do not generate a @detailmenu in the "Top" node's menu.
# texinfo_no_detailmenu = False


# -- Options for Epub output ----------------------------------------------

# Bibliographic Dublin Core info.
epub_title = project
epub_author = author
epub_publisher = author
epub_copyright = copyright

# The basename for the epub file. It defaults to the project name.
# epub_basename = project

# The HTML theme for the epub output. Since the default themes are not optimized
# for small screen space, using the same theme for HTML and epub output is
# usually not wise. This defaults to 'epub', a theme designed to save visual
# space.
# epub_theme = 'epub'

# The language of the text. It defaults to the language option
# or 'en' if the language is not set.
# epub_language = ''

# The scheme of the identifier. Typical schemes are ISBN or URL.
# epub_scheme = ''

# The unique identifier of the text. This can be a ISBN number
# or the project homepage.
# epub_identifier = ''

# A unique identification for the text.
# epub_uid = ''

# A tuple containing the cover image and cover page html template filenames.
# epub_cover = ()

# A sequence of (type, uri, title) tuples for the guide element of content.opf.
# epub_guide = ()

# HTML files that should be inserted before the pages created by sphinx.
# The format is a list of tuples containing the path and title.
# epub_pre_files = []

# HTML files shat should be inserted after the pages created by sphinx.
# The format is a list of tuples containing the path and title.
# epub_post_files = []

# A list of files that should not be packed into the epub file.
epub_exclude_files = ['search.html']

# The depth of the table of contents in toc.ncx.
# epub_tocdepth = 3

# Allow duplicate toc entries.
# epub_tocdup = True

# Choose between 'default' and 'includehidden'.
# epub_tocscope = 'default'

# Fix unsupported image types using the Pillow.
# epub_fix_images = False

# Scale large images.
# epub_max_image_width = 0

# How to display URL addresses: 'footnote', 'no', or 'inline'.
# epub_show_urls = 'inline'

# If false, no index is generated.
# epub_use_index = True


# Example configuration for intersphinx: refer to the Python standard library.
intersphinx_mapping = {'https://docs.python.org/': None}



import os

from pm4ngs.jupyterngsplugin.image.pdftobase64 import pdftobase64
from pm4ngs.jupyterngsplugin.utils.load_content_dict import load_content_dict


def rseqc_plot_table(sample_list, data_path, width, height):
    """

    :param sample_list: list of samples prefix names
    :param data_path:
    :param width:
    :param height:
    :return:
    """
    str_msg = 'Click on figure to retrieve original PDF file\n\n'
    str_msg += '| Sample | Boxplot | HeatMap | Splice Events | Junction Saturation '
    str_msg += '|\n| --- | --- | --- | --- | --- '
    str_msg += '|\n'
    for s in sample_list:
        str_msg += '| ' + s
        str_msg += '| '
        files = [f for ds, dr, files in os.walk(data_path) for f in files if
                 f.startswith(s) and f.endswith('rseqc.qual.boxplot.pdf')
                 and os.path.getsize(os.path.join(data_path, f)) != 0]
        if len(files) == 1:
            f = os.path.relpath(os.path.join(data_path, files[0]))
            str_msg += ' <a href="'
            str_msg += f.replace(' ', '%20')
            str_msg += '" target="_blank">'
            str_msg += '<img src="data:image/png;base64,' + pdftobase64(f, width, height) + '">'
            str_msg += '</a> |'
        else:
            str_msg += ' --- |'
        files = [f for ds, dr, files in os.walk(data_path) for f in files if
                 f.startswith(s) and f.endswith('rseqc.qual.heatmap.pdf')
                 and os.path.getsize(os.path.join(data_path, f)) != 0]
        if len(files) == 1:
            f = os.path.relpath(os.path.join(data_path, files[0]))
            str_msg += ' <a href="'
            str_msg += f.replace(' ', '%20')
            str_msg += '" target="_blank">'
            str_msg += '<img src="data:image/png;base64,' + pdftobase64(f, width, height) + '">'
            str_msg += '</a> |'
        else:
            str_msg += ' --- |'
        files = [f for ds, dr, files in os.walk(data_path) for f in files if
                 f.startswith(s) and f.endswith('rseqc.splice_events.pdf')
                 and os.path.getsize(os.path.join(data_path, f)) != 0]
        if len(files) == 1:
            f = os.path.relpath(os.path.join(data_path, files[0]))
            str_msg += ' <a href="'
            str_msg += f.replace(' ', '%20')
            str_msg += '" target="_blank">'
            str_msg += '<img src="data:image/png;base64,' + pdftobase64(f, width, height) + '">'
            str_msg += '</a> |'
        else:
            str_msg += ' --- |'
        files = [f for ds, dr, files in os.walk(data_path) for f in files if
                 f.startswith(s) and f.endswith('rseqc.junctionSaturation_plot.pdf')
                 and os.path.getsize(os.path.join(data_path, f)) != 0]
        if len(files) == 1:
            f = os.path.relpath(os.path.join(data_path, files[0]))
            str_msg += ' <a href="'
            str_msg += f.replace(' ', '%20')
            str_msg += '" target="_blank">'
            str_msg += '<img src="data:image/png;base64,' + pdftobase64(f, width, height) + '">'
            str_msg += '</a> |'
        else:
            str_msg += ' --- |'
        str_msg += '\n'
    return str_msg


def rseqc_table(sample_list, data_path):
    """
    Create RseQC report table
    :param sample_list: list of samples prefix names
    :param data_path: Folder with RseQC outputs
    :return:
    """
    str_msg = '| Sample | Total records | QC failed | Optical/PCR duplicate '
    str_msg += '| mapq < mapq_cut (non-unique) | mapq >= mapq_cut (unique) '
    str_msg += '| Reads map to \'+\' '
    str_msg += '| Reads map to \'-\' '
    str_msg += '| Non-splice reads '
    str_msg += '| Splice reads '
    str_msg += '|\n| --- | --- | --- | --- | --- | --- | --- | --- | --- | --- '
    str_msg += '|\n'
    for s in sample_list:
        str_msg += '| ' + s
        str_msg += '| '

        files = [f for ds, dr, files in os.walk(data_path) for f in files if
                 f.startswith(s) and f.endswith('.bam_stat.txt')
                 and os.path.getsize(os.path.join(data_path, f)) != 0]
        if len(files) == 1:
            f = os.path.relpath(os.path.join(data_path, files[0]))
            stats = load_content_dict(f, ':', True)
            str_msg += "{:,}".format(int(stats['Total records'])) + ' |'
            str_msg += "{:,}".format(int(stats['QC failed'])) + ' |'
            str_msg += "{:,}".format(int(stats['Optical/PCR duplicate'])) + ' |'
            str_msg += "{:,}".format(int(stats['mapq < mapq_cut (non-unique)'])) + ' |'
            str_msg += "{:,}".format(int(stats['mapq >= mapq_cut (unique)'])) + ' |'
            str_msg += "{:,}".format(int(stats['Reads map to \'+\''])) + ' |'
            str_msg += "{:,}".format(int(stats['Reads map to \'-\''])) + ' |'
            str_msg += "{:,}".format(int(stats['Non-splice reads'])) + ' |'
            str_msg += "{:,}".format(int(stats['Splice reads'])) + ' |'
        else:
            str_msg += ' | --- | --- | --- | --- | --- | --- | --- | --- | --- '
        str_msg += '\n'
    return str_msg





def load_content_dict(file, delimiter, strip_line=True,
                      replace_old=None, replace_new=None, comment=None,
                      startswith=None):
    """
    This function load file content in a dictionary spliting the line in two (key-value)
    :param file: File to parse
    :param delimiter: Delimiter to split line
    :param strip_line: Strip line before splitting
    :param replace_old: Replace substring  replace_old with replace_new before splitting
    :param replace_new: Replace substring  replace_old with replace_new before splitting
    :param comment: Comment str to exclude line
    :param startswith: Parse line if start with startswith
    :return: Dict with file content
    """
    result = {}
    with open(file) as fin:
        for line in fin:
            if (not comment or (comment and not line.startswith(comment))) or \
                    (startswith and line.startswith(startswith)):
                if strip_line:
                    line = line.strip()
                if replace_old:
                    line = line.replace(replace_old, replace_new)
                fields = line.split(delimiter)
                if len(fields) == 2:
                    result[fields[0].strip()] = fields[1].strip()
    return result


def load_content_dict_line(file, delimiter, startswith, sec_delimiter, strip_line=True,
                           replace_old=None, replace_new=None):
    """
    This function load file content in a dictionary spliting the line in two (key-value)
    :param file: File to parse
    :param delimiter: Delimiter to split line
    :param strip_line: Strip line before splitting
    :param replace_old: Replace substring  replace_old with replace_new before splitting
    :param replace_new: Replace substring  replace_old with replace_new before splitting
    :param startswith: Parse line if start with startswith
    :return: Dict with file content
    """
    result = {}
    with open(file) as fin:
        for line in fin:
            if line.startswith(startswith):
                if strip_line:
                    line = line.strip()
                if replace_old:
                    line = line.replace(replace_old, replace_new)
                fields = line.split(delimiter)
                if len(fields) == 2:
                    result[fields[0].strip()] = fields[1].strip().split(sec_delimiter)[0]
    return result



import os

from pm4ngs.jupyterngsplugin.utils.yaml_utils import write_to_yaml


def write_sample_table_pe_to_yaml(sample_table, data_dir, file_name):
    samples = []
    for i, r in sample_table.iterrows():
        sample = []
        if not r['file']:
            r1 = os.path.join(data_dir, r['sample_name'] + '_1.fastq.gz')
            r2 = os.path.join(data_dir, r['sample_name'] + '_2.fastq.gz')
            sample.append({'class': 'File', 'path': r1})
            sample.append({'class': 'File', 'path': r2})
        else:
            for f in r['file'].split(','):
                sample.append({'class': 'File', 'path': r1})
        samples.append(sample)
    write_to_yaml(samples, file_name)


def write_sample_table_se_to_yaml(sample_table, data_dir, file_name):
    samples = []
    for i, r in sample_table.iterrows():
        if not r['file']:
            r = os.path.join(data_dir, r['sample_name'] + '.fastq.gz')
            samples.append({'class': 'File', 'path': r})
        else:
            samples.append({'class': 'File', 'path': r['file']})
    write_to_yaml(samples, file_name)



import os
import zipfile


def parse_fastqc_zip(zip_file):
    """
    This function extract from a fastQC zip file the failing tests and data
    :param zip_file: FastQC zip file
    :return:
    """
    tests = {}
    tot_seq = 0
    poor_quality = 0
    seq_len = ''
    gc_content = ''
    if os.path.exists(zip_file) and os.path.getsize(zip_file) != 0:
        prefix = os.path.basename(zip_file).replace('.zip', '')
        with zipfile.ZipFile(zip_file) as myzip:
            with myzip.open(prefix + '/summary.txt') as myfile:
                for line in myfile:
                    fields = line.strip().decode('utf8').split('\t')
                    tests[fields[1]] = fields[0]
            with myzip.open(prefix + '/fastqc_data.txt') as myfile:
                for line in myfile:
                    fields = line.strip().decode('utf8').split('\t')
                    if fields[0] == 'Total Sequences':
                        tot_seq = int(fields[1])
                    if fields[0] == 'Sequences flagged as poor quality':
                        poor_quality = int(fields[1])
                    if fields[0] == 'Sequence length':
                        seq_len = fields[1]
                    if fields[0] == '%GC':
                        gc_content = fields[1]
    return tests, tot_seq, poor_quality, seq_len, gc_content



import json
import os

import yaml
from cookiecutter.exceptions import FailedHookException, OutputDirExistsException
from cookiecutter.main import cookiecutter


def execute_cookiecutter(template, config_file, sample_table, copy_rawdata):
    extra_context = {
        'default_context': {}
    }
    no_input = False
    if config_file:
        print('Using config file: {}'.format(config_file))
        with open(config_file) as file:
            extra_context = yaml.load(file, Loader=yaml.FullLoader)
            print(json.dumps(extra_context['default_context'], indent=4))
            no_input = True
    if os.path.exists(sample_table):
        os.environ['PM4NGS_SAMPLE_TABLE'] = os.path.abspath(sample_table)
        os.environ['PM4NGS_COPY_RAWDATA'] = str(copy_rawdata)
        os.environ['PM4NGS_WORK_DIR'] = os. getcwd()
        try:
            cookiecutter(template,
                         no_input=no_input,
                         extra_context=extra_context['default_context'])
        except FailedHookException as e:
            print(e)
        except OutputDirExistsException as e:
            print(e)
    else:
        print('Sample table file {} not found.'.format(sample_table))



import os


def count_lines(filename, comment):
    """
    Count lines in a file excluding the comment
    :param filename: Path to the file to count
    :param comment: Comment char or string at the beginning of the line to exclude
    :return:
    """
    count = 0
    if os.path.exists(filename) and os.path.getsize(filename) != 0:
        with open(filename, 'r') as fin:
            for line in fin:
                if not line.startswith(comment):
                    count += 1
    return count



import itertools
import os

from pm4ngs.jupyterngsplugin.markdown.utils import get_link_image
from pm4ngs.jupyterngsplugin.markdown.utils import get_link_name
from pm4ngs.jupyterngsplugin.markdown.utils import get_link_text
from pm4ngs.jupyterngsplugin.utils.count_lines import count_lines


def diffbind_table(sample_table, result_dir, width, height):
    """
    Diffbid report table
    :param output_suffix: Result path
    :param width: Image width
    :param height: Image height
    :return: string
    """
    comparisons = []
    for s in itertools.combinations(sample_table['condition'].unique(), 2):
        comparisons.append(list(s))
    str_msg = ''
    for c in comparisons:
        comp = '{0}_vs_{1}'.format(c[0], c[1])
        str_msg += '### Condition: {0} vs {1}\n\n'.format(c[0], c[1])
        str_msg += '| Deseq2 | EdgeR |\n'
        str_msg += '| --- | --- |\n'
        f1 = os.path.relpath(os.path.join(result_dir, 'condition_'
                                          + comp + '_diffbind_deseq2_plot.png'))
        f2 = os.path.relpath(os.path.join(result_dir, 'condition_'
                                          + comp + '_diffbind_edgeR_plot.png'))
        str_msg += '| ' + get_link_image(f1, width, height, ' --- ')
        str_msg += '| ' + get_link_image(f2, width, height, ' --- ')
        str_msg += '|\n'
        f1 = os.path.relpath(os.path.join(result_dir, 'condition_'
                                          + comp + '_diffbind_deseq2_plotHeatmap.png'))
        f2 = os.path.relpath(os.path.join(result_dir, 'condition_'
                                          + comp + '_diffbind_edgeR_plotHeatmap.png'))
        str_msg += '| ' + get_link_image(f1, width, height, ' --- ')
        str_msg += '| ' + get_link_image(f2, width, height, ' --- ')
        str_msg += '|\n'
        f1 = os.path.relpath(os.path.join(result_dir, 'condition_'
                                          + comp + '_diffbind_deseq2_plotMA.png'))
        f2 = os.path.relpath(os.path.join(result_dir, 'condition_'
                                          + comp + '_diffbind_edgeR_plotMA.png'))
        str_msg += '| ' + get_link_image(f1, width, height, ' --- ')
        str_msg += '| ' + get_link_image(f2, width, height, ' --- ')
        str_msg += '|\n'
        f1 = os.path.relpath(os.path.join(result_dir, 'condition_'
                                          + comp + '_diffbind_deseq2_plotVolcano.png'))
        f2 = os.path.relpath(os.path.join(result_dir, 'condition_'
                                          + comp + '_diffbind_edgeR_plotVolcano.png'))
        str_msg += '| ' + get_link_image(f1, width, height, ' --- ')
        str_msg += '| ' + get_link_image(f2, width, height, ' --- ')
        str_msg += '|\n'
        f1 = os.path.relpath(os.path.join(result_dir, 'condition_'
                                          + comp + '_diffbind_deseq2_plotPCA.png'))
        f2 = os.path.relpath(os.path.join(result_dir, 'condition_'
                                          + comp + '_diffbind_edgeR_plotPCA.png'))
        str_msg += '| ' + get_link_image(f1, width, height, ' --- ')
        str_msg += '| ' + get_link_image(f2, width, height, ' --- ')
        str_msg += '|\n'
        f1 = os.path.relpath(os.path.join(result_dir, 'condition_'
                                          + comp + '_diffbind_deseq2_plotPCA_contrast.png'))
        f2 = os.path.relpath(os.path.join(result_dir, 'condition_'
                                          + comp + '_diffbind_edgeR_plotPCA_contrast.png'))
        str_msg += '| ' + get_link_image(f1, width, height, ' --- ')
        str_msg += '| ' + get_link_image(f2, width, height, ' --- ')
        str_msg += '|\n'
        f1 = os.path.relpath(os.path.join(result_dir, 'condition_'
                                          + comp + '_diffbind_deseq2_plotBox.png'))
        f2 = os.path.relpath(os.path.join(result_dir, 'condition_'
                                          + comp + '_diffbind_edgeR_plotBox.png'))
        str_msg += '| ' + get_link_image(f1, width, height, ' --- ')
        str_msg += '| ' + get_link_image(f2, width, height, ' --- ')
        str_msg += '|\n\n'

    return str_msg


def peak_calling_table_with_qc(factors, alignment_path, peak_calling_path, width, height):
    """
    Create a peck calling table with phantompeakqualtools plots
    :param factors: Pandas DataFrame with the samples and conditions
    :param alignment_path: Path to the alignments results
    :param peak_calling_path: Path to the peak calling results
    :param width: Image width
    :param height: Image height
    :return: An string in markdown language
    """
    conditions = factors['condition'].unique()
    n_max_samples = 0
    for c in conditions:
        rep = factors[factors['condition'] == c]['sample_name']
        if len(rep) > n_max_samples:
            n_max_samples = len(rep)

    str_msg = '| Condition '
    str_msg += '| No. Peaks '
    for i in range(0, n_max_samples):
        str_msg += '| Sample {0} '.format(i)
    str_msg += '|\n'
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '| --- '
    str_msg += '|\n'

    for c in conditions:
        str_msg += '| ' + c
        str_msg += '| '
        r0 = os.path.relpath(os.path.join(peak_calling_path, c + '_R0_peaks.narrowPeak'))
        if os.path.exists(r0) and os.path.getsize(r0) != 0:
            str_msg += get_link_text(r0, count_lines(r0, '#'), ' --- ')
            str_msg += '|'
        else:
            str_msg += ' --- |'
        rep = factors[factors['condition'] == c]['sample_name']
        for r in rep:
            f = os.path.relpath(os.path.join(alignment_path, r + '_sorted.tagAlign.cc.plot.pdf'))
            str_msg += get_link_image(f, width, height, ' --- ')
            str_msg += '|'
        if len(rep) < n_max_samples:
            for i in range(len(rep), n_max_samples):
                str_msg += ' --- |'
        str_msg += '\n'
    str_msg += '\n'
    str_msg += '\nClick on figure to retrieve original PDF file\n\n'

    return str_msg


def idr_table(factors, idr_path, width, height):
    str_msg = ''
    for c in factors['condition'].unique():
        str_msg += '\n## Condition: {0}\n\n'.format(c)
        f = os.path.relpath(os.path.join(idr_path, c + '.narrowPeak_annotation.txt'))
        str_msg += 'Annotated peak file: '
        str_msg += get_link_name(f, ifempty="")
        str_msg += '\n\nIDR peaks: ' + str(count_lines(f, '#'))
        str_msg += '\n\n| Default  peaks | No alternate<br>summit peaks |\n'
        str_msg += '| --- | --- |\n'
        f = os.path.relpath(os.path.join(idr_path, c + '.narrowPeak.png'))
        str_msg += ' | ' + get_link_image(f, width, height, ' --- ')
        f = os.path.relpath(os.path.join(idr_path, c + '.narrowPeak.noalternatesummitpeaks.png'))
        str_msg += ' | ' + get_link_image(f, width, height, ' --- ')
        str_msg += ' |\n\n'

    return str_msg



import os
import os.path

from setuptools import find_packages
from setuptools import setup


def readme():
    with open(os.path.join(os.path.dirname(__file__), 'README.md')) as f:
        return f.read()

# Set __version__
exec(open('src/pm4ngs/__init__.py').read())

setup(
    name='pm4ngs',
    packages=find_packages(where='src'),
    package_dir={
        '': 'src',
    },
    data_files=[('', ['README.md'])],
    use_scm_version=True,
    setup_requires=['wheel', 'setuptools_scm'],
    description='PM4NGS generates a standard organizational structure for Next '
                'Generation Sequencing (ngs) data analysis ',
    long_description=readme(),
    long_description_content_type='text/markdown',
    license='Public Domain',
    author='Vera Alvarez, Roberto',
    author_email='veraalva' '@' 'ncbi.nlm.nih.gov',
    maintainer='Vera Alvarez, Roberto',
    maintainer_email='veraalva' '@' 'ncbi.nlm.nih.gov',
    url='https://github.com/ncbi/pm4ngs',
    install_requires=['bioconda2biocontainer',
                      'biopython',
                      'cookiecutter',
                      'cwltool',
                      'docker',
                      'galaxy-tool-util',
                      'GitPython',
                      'goenrichment',
                      'jupyter',
                      'matplotlib',
                      'networkx',
                      'numpy',
                      'pandas',
                      'PyYAML',
                      'requests',
                      'scipy',
                      'seaborn',
                      'statsmodels',
                      'urllib3',
                      'xmltodict'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Science/Research',
        'License :: Public Domain',
        'Natural Language :: English',
        'Operating System :: Unix',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Scientific/Engineering :: Bio-Informatics'
    ],
    keywords='Biocontainers',
    project_urls={
        'Documentation': 'https://pm4ngs.readthedocs.io/',
        'Source': 'https://github.com/ncbi/pm4ngs',
        'Tracker': 'https://github.com/ncbi/pm4ngs/issues',
    },
    entry_points={
        'console_scripts': [
            'pm4ngs-server = pm4ngs.main:start_server_main',
            'pm4ngs-create = pm4ngs.main:create_project',
            'pm4ngs-rnaseq = pm4ngs.main:rnaseq',
            'pm4ngs-rnaseq-demo = pm4ngs.main:rnaseq_demo',
            'pm4ngs-chipseq = pm4ngs.main:chipseq',
            'pm4ngs-chipseq-demo = pm4ngs.main:chipseq_demo',
            'pm4ngs-chipexo = pm4ngs.main:chipexo',
            'pm4ngs-chipexo-demo = pm4ngs.main:chipexo_demo'
        ],
    }
)



import os

from pm4ngs.jupyterngsplugin.image.pdftobase64 import pdftobase64
from pm4ngs.jupyterngsplugin.image.pngtobase64 import imagetobase64


def hide_code_str():
    """
    Print an HTML code to hide python code in Jupyter notebooks
    :return: String
    """
    return '''
    <script>
        code_show=true;
        function code_toggle() {
            if (code_show){
                $('div.input').hide();
            } else {
                $('div.input').show();
            }
            code_show = !code_show
        }
        $( document ).ready(code_toggle);
    </script>
    The raw code for this IPython notebook is by default hidden for easier reading.
    To toggle on/off the raw code, click <a href="javascript:code_toggle()">here</a>.
    '''


def get_link_size(filename, size_unit="MB", ifempty=""):
    """
    Print an HTML link for the file using the size as text
    :param filename: file to be linked
    :param size_unit: Size units to display KB, MG or GB.
           It will display bytes for any other option
    :param ifempty: Use this if the output is empty
    :return: HTML string
    """
    str_msg = ifempty
    if os.path.exists(filename):
        size = os.path.getsize(filename)
        if size != 0:
            if size_unit == "KB":
                size = size / 1024.0
            elif size_unit == "MB":
                size = size / (1024.0 ** 2)
            elif size_unit == "GB":
                size = size / (1024.0 ** 3)
            str_msg = ' <a href="'
            str_msg += filename.replace(' ', '%20')
            str_msg += '" target="_blank">'
            str_msg += "{0:.2f}".format(size)
            str_msg += size_unit
            str_msg += '</a>'
    return str_msg


def get_link_image(filename, width, height, ifempty=''):
    """
    Print an HTML link for the file using ta reduce image
    :param filename: file to be linked
    :param width: Image width
    :param height: Image height
    :param ifempty: Use this if the output is empty
    :return: HTML string
    """
    str_msg = ifempty
    if os.path.exists(filename) and os.path.getsize(filename) != 0:
        str_msg = ' <a href="'
        str_msg += filename.replace(' ', '%20')
        str_msg += '" target="_blank">'
        if filename.endswith('.pdf'):
            str_msg += '<img src="data:image/png;base64,' + pdftobase64(filename, width, height)
        else:
            str_msg += '<img src="data:image/png;base64,' + imagetobase64(filename, width, height)
        str_msg += '">'
    return str_msg


def get_link_name(filename, ifempty=""):
    """
    Print an HTML link for the file using the basename as text
    :param filename: file to be linked
    :param ifempty: Use this if the output is empty
    :return: HTML string
    """
    str_msg = ifempty
    if os.path.exists(filename) and os.path.getsize(filename) != 0:
        str_msg = ' <a href="'
        str_msg += filename.replace(' ', '%20')
        str_msg += '" target="_blank">'
        str_msg += os.path.basename(filename)
        str_msg += '</a>'
    return str_msg


def get_link_text(filename, text, ifempty=""):
    """
    Print an HTML link for the file using the basename as text
    :param filename: file to be linked
    :param text: Text to use in the link
    :param ifempty: Use this if the output is empty
    :return: HTML string
    """
    str_msg = ifempty
    if os.path.exists(filename) and os.path.getsize(filename) != 0:
        str_msg = ' <a href="'
        str_msg += filename.replace(' ', '%20')
        str_msg += '" target="_blank">'
        str_msg += '{0}'.format(text)
        str_msg += '</a>'
    return str_msg


def find_file_print_link_size(folder_path, prefix, sufix, size_unit, ifempty):
    """
    Find file in a folder and print link with size
    :param folder_path: Folder containing the file
    :param prefix: File name prefix
    :param sufix: File name sufix
    :param size_unit: Size units to display KB, MG or GB.
           It will display bytes for any other option
    :param ifempty: Use this if the output is empty
    :return:
    """
    files = [f for ds, dr, files in os.walk(folder_path)
             for f in files if f.startswith(prefix) and f.endswith(sufix)
             and os.path.getsize(os.path.join(folder_path, f)) != 0]
    if len(files) == 1:
        f = os.path.relpath(os.path.join(folder_path, files[0]))
        return get_link_size(f, size_unit, ifempty)
    return ifempty


def find_file_print_link_name(folder_path, prefix, sufix, size_unit, ifempty):
    """
    Find file in a folder and print link with name
    :param folder_path: Folder containing the file
    :param prefix: File name prefix
    :param sufix: File name sufix
    :param size_unit: Size units to display KB, MG or GB.
           It will display bytes for any other option
    :param ifempty: Use this if the output is empty
    :return:
    """
    files = [f for ds, dr, files in os.walk(folder_path)
             for f in files if f.startswith(prefix) and f.endswith(sufix)
             and os.path.getsize(os.path.join(folder_path, f)) != 0]
    if len(files) == 1:
        f = os.path.relpath(os.path.join(folder_path, files[0]))
        return get_link_name(f, ifempty)
    return ifempty


def info_table(notebook, result_dir):
    str_msg = '#### Info\n\n'
    str_msg += '| Type | Link |\n'
    str_msg += '| --- | --- |\n'
    str_msg += '| Notebook | <a href="' \
               + notebook.replace(' ', '%20') \
               + '.ipynb" target="_blank">' \
               + notebook + '</a> | \n'
    str_msg += '| Results | <a href="' \
               + os.path.relpath(result_dir).replace(' ', '%20') \
               + '" target="_blank">' \
               + os.path.basename(result_dir) + '</a> |\n\n'

    return str_msg



import subprocess
from IPython.display import Javascript


def save_2_html(nb_name):
    script = '''
    require(["base/js/namespace"],function(Jupyter) {
        Jupyter.notebook.save_checkpoint();
    });
    '''

    Javascript(script)
    subprocess.run(["jupyter", "nbconvert", "--to", "html", nb_name])
